//
//  ActiveChatViewController.m
//  rrrt
//
//  Created by Silvia Cabezas Nieto on 10/05/15.
//  Copyright (c) 2015 Silvia Cabezas Nieto. All rights reserved.
//

#import "ActiveChatViewController.h"
#import "TFCChat.h"
#import "TFCTableCell.h"

@interface ActiveChatViewController ()

@end

@implementation ActiveChatViewController

@synthesize tView;

- (AppDelegate *)appDelegate {
    return (AppDelegate *)[[UIApplication sharedApplication] delegate];
}

- (XMPPStream *)xmppStream {
    return [[self appDelegate] xmppStream];
}

- (ChatsManager *)chatDelegate
{
    return [[self appDelegate] chatsDelegate];
}

- (NSMutableArray *)chats
{
    return [[self chatDelegate] chats];
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.tView.delegate = self;
    self.tView.dataSource = self;
    [self subscribeToChat];
    [self.tView reloadData];
}


- (void)subscribeToChat
{
    self->sourceChat = [[self chatDelegate] setActiveChatSubscription:[[self appDelegate] activeUserChat] theView:self];
    [self.tView reloadData];
}

#pragma mark -
#pragma mark Table view delegates

static CGFloat padding = 20.0;

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    TFCMessage *oMsg = (TFCMessage *) [self->sourceChat.messages objectAtIndex:indexPath.row];
    
    static NSString *CellIdentifier = @"MessageCellIdentifier";
    
    TFCTableCell *cell = (TFCTableCell *)[tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    
    if (cell == nil) {
        cell = [[TFCTableCell alloc] initWithFrame:CGRectZero reuseIdentifier:CellIdentifier];
    }

    
    NSString *sender = oMsg.user;
    NSString *message = oMsg.content;
    NSString *time = [oMsg getTimestamp];
    
    CGSize  textSize = { 260.0, 10000.0 };
    CGSize size = [message sizeWithFont:[UIFont boldSystemFontOfSize:13] constrainedToSize:textSize lineBreakMode:UILineBreakModeWordWrap];
    //size.width += (padding/2);
    
    //CGSize size = [message sizeWithAttributes:@{NSFontAttributeName: [UIFont systemFontOfSize:13.0f]}];
    
    // Values are fractional -- you should take the ceilf to get equivalent values
    //size = CGSizeMake(ceilf(size.width), ceilf(size.height));
    
    
    
    size.width += (padding/2);
    size.height += (padding/2);
    
    cell.messageContentView.text = message;
    cell.accessoryType = UITableViewCellAccessoryNone;
    cell.userInteractionEnabled = NO;
    
    UIImage *bgImage = nil;
    
    
    if ([sender isEqualToString:@"me"]) { // left aligned
        
        bgImage = [[UIImage imageNamed:@"orange.png"] stretchableImageWithLeftCapWidth:24  topCapHeight:15];
        
        [cell.messageContentView setFrame:CGRectMake(padding, padding*2, size.width, size.height)];
        
        [cell.bgImageView setFrame:CGRectMake(cell.messageContentView.frame.origin.x - padding/2,
                                              cell.messageContentView.frame.origin.y - padding/2,
                                              size.width+padding,
                                              size.height+padding)];
        
    } else {
        
        bgImage = [[UIImage imageNamed:@"aqua.png"] stretchableImageWithLeftCapWidth:24  topCapHeight:15];
        
        [cell.messageContentView setFrame:CGRectMake(320 - size.width - padding,
                                                     padding*2,
                                                     size.width,
                                                     size.height)];
        
        [cell.bgImageView setFrame:CGRectMake(cell.messageContentView.frame.origin.x - padding/2, 
                                              cell.messageContentView.frame.origin.y - padding/2, 
                                              size.width+padding, 
                                              size.height+padding)];
        
    }
    
    cell.bgImageView.image = bgImage;
    cell.senderAndTimeLabel.text = [NSString stringWithFormat:@"%@ %@", sender, time];
    
    [cell sizeToFit];
    
    
    CGRect rect = cell.frame;
    rect.size.height += padding*2;
    cell.frame = rect;
    
    return cell;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    
    return [self->sourceChat getNumberOfMessages];
    
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    
    return 1;
    
}

- (void)NewMessage:(TFCMessage *)oMessage
{
    [self.tView reloadData];
}

-(IBAction)Cerrar
{
    AppDelegate *del = [self appDelegate];
    [[self chatDelegate] clearActiveChatSubscription:del.activeUserChat];
    del.activeUserChat = nil;
    self->sourceChat = nil;
    [self performSegueWithIdentifier:@"CloseChat" sender:self];
}

-(IBAction)Enviar
{
    NSString *messageStr = self.messageField.text;
    
    if([messageStr length] > 0) {
        
        NSXMLElement *body = [NSXMLElement elementWithName:@"body"];
        [body setStringValue:messageStr];
        
        NSXMLElement *message = [NSXMLElement elementWithName:@"message"];
        [message addAttributeWithName:@"type" stringValue:@"chat"];
        [message addAttributeWithName:@"to" stringValue:self->sourceChat.user];
        [message addChild:body];
        
        [self.xmppStream sendElement:message];
        
        self.messageField.text = @"";
        
        TFCMessage *oMsg = [[TFCMessage alloc] initWithContentAndUser:messageStr theUser:@"me"];
        [oMsg Readed];
        [self->sourceChat AddMessage:oMsg];
        [self.tView reloadData];
    }
    
    NSIndexPath *topIndexPath = [NSIndexPath indexPathForRow:[self->sourceChat getNumberOfMessages]-1
                                                   inSection:0];
    
    [self.tView scrollToRowAtIndexPath:topIndexPath
                      atScrollPosition:UITableViewScrollPositionMiddle
                              animated:YES];
    [self.tView reloadData];
}


@end
