//
//  AppDelegate.m
//  rrrt
//
//  Created by Silvia Cabezas Nieto on 09/05/15.
//  Copyright (c) 2015 Silvia Cabezas Nieto. All rights reserved.
//

#import "AppDelegate.h"


@interface AppDelegate()

@property XMPPRosterMemoryStorage *rosterstorage;

- (void)initializeRoster;
- (void)setupStream;
- (NSString *)getUserString:(NSString *)userName;

- (void)goOnline;
- (void)goOffline;

@end

@implementation AppDelegate

@synthesize window;

@synthesize xmppStream;
@synthesize xmppRoster;
@synthesize rosterstorage;

@synthesize mainConfig;

@synthesize contactsDelegate;
@synthesize contactsViewDelegate;

@synthesize chatsDelegate;
@synthesize chatsViewDelegate;

@synthesize activeUserChat;

//************************
//APPLICATION LEVEL EVENTS
//************************

//Disconnect and reconect as the application becomes active and inactive.

- (void)applicationWillResignActive:(UIApplication *)application {
    [self disconnect];
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
    if (self.didOnlineByLoggin)
    {
        [self connect];
    }
}

//************************
//ENCAPSULATION METHODS
//************************

-(TFCConfig *)mainConfig
{
    if (mainConfig == NULL) mainConfig = [[TFCConfig alloc] init];
    return mainConfig;
}

//************************
//PUBLIC/PRIVATE METHODS
//************************

-(void)DoAuthenticate
{
    isOpen = YES;
    NSError *error = nil;
    if ([password isEqualToString:@""])
    {
        [[self xmppStream] authenticateWithPassword:@"devil" error:&error];
    }
    else
    {
        [[self xmppStream] authenticateWithPassword:password error:&error];
    }

}

//Initializes the roster object
- (void)initializeRoster
{
    self.rosterstorage = [[XMPPRosterMemoryStorage alloc] init];
    self.xmppRoster = [[XMPPRoster alloc] initWithRosterStorage:rosterstorage dispatchQueue:dispatch_get_main_queue()];
    xmppRoster.autoAcceptKnownPresenceSubscriptionRequests = true;
    xmppRoster.autoFetchRoster = true;
    [xmppRoster activate:xmppStream];
    [xmppRoster fetchRoster];
}

//Initializes the stream object
- (void)setupStream {
    xmppStream = [[XMPPStream alloc] init];
    //[xmppStream sethostName:@"localhost"];
    [xmppStream setHostName:[self mainConfig].serverHost];
    [xmppStream addDelegate:self delegateQueue:dispatch_get_main_queue()];
}

//Converts user name to a string that includes virtual host.
- (NSString *)getUserString:(NSString *)userName
{
    NSArray *arr= [[NSArray alloc]initWithObjects:userName,[self mainConfig].serverFullName, nil];
    NSString *userString = [arr componentsJoinedByString:@"@"];
    return userString;
}

//Helper function to send presence
- (void)goOnline {
    XMPPPresence *presence = [XMPPPresence presence];
    [[self xmppStream] sendElement:presence];
}

//Helper function to send presence
- (void)goOffline {
    XMPPPresence *presence = [XMPPPresence presenceWithType:@"unavailable"];
    [[self xmppStream] sendElement:presence];
}

-(BOOL)connectForLogin
{
    self->connectionStartedByLogin = YES;
    return [self connect];
}

-(BOOL)connectForRegister
{
    self->connectionStartedByLogin = NO;
    return [self connect];
}

//Public mthod that connects the stream. Called by the LoginView
- (BOOL)connect {
    
    self.canRegisterUser = false;
    
    [self setupStream];
    
    NSString *jabberID = [[NSUserDefaults standardUserDefaults] stringForKey:@"userID"];
    NSString *myPassword = [[NSUserDefaults standardUserDefaults] stringForKey:@"userPassword"];
    
    if (![xmppStream isDisconnected])
    {
        if (jabberID == nil || myPassword == nil)
        {
            password = myPassword;
            [self DoAuthenticate];
            return YES;
        }
        return NO;
    }
    
    NSError *error = nil;
    
    if (jabberID == nil || myPassword == nil)
    {
        password = @"";
        [xmppStream setMyJID:[XMPPJID jidWithString:[self getUserString:jabberID]]];
        if (![xmppStream connectWithTimeout:XMPPStreamTimeoutNone error:&error])
        {
            UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"Error"
                                                                message:[NSString stringWithFormat:@"Can't connect to server %@", [error localizedDescription]]
                                                               delegate:nil
                                                      cancelButtonTitle:@"Ok"
                                                      otherButtonTitles:nil];
            [alertView show];
            
            return NO;
        }
        return YES;
    }
    
    [xmppStream setMyJID:[XMPPJID jidWithString:[self getUserString:jabberID]]];
    
    password = myPassword;
    
    
    if (![xmppStream connectWithTimeout:XMPPStreamTimeoutNone error:&error])
    {
        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"Error"
                                                            message:[NSString stringWithFormat:@"Can't connect to server %@", [error localizedDescription]]
                                                           delegate:nil
                                                  cancelButtonTitle:@"Ok"
                                                  otherButtonTitles:nil];
        [alertView show];
        
        return NO;
    }
    
    return YES;
}

//Public method for closing conneciton
- (void)disconnect {
    
    [self goOffline];
    [xmppStream disconnect];
    [contactsDelegate didDisconnect];
    [contactsViewDelegate didDisconnect];
}

- (void)dealloc {
    
    [xmppStream removeDelegate:self];
    [xmppStream disconnect];
    
}



//************************
//XMMSTRESM LEVEL EVENTS
//************************

#pragma mark -
#pragma mark XMPP delegates

//Socket connected
- (void)xmppStreamDidConnect:(XMPPStream *)sender {
    
    [self DoAuthenticate];
    
}

//Credentials accepted
- (void)xmppStreamDidAuthenticate:(XMPPStream *)sender {
    if (self->connectionStartedByLogin)
    {
        self.contactsDelegate = [[ContactsManager alloc] init];
        self.chatsDelegate = [[ChatsManager alloc] init];
        [self initializeRoster];
        [self goOnline];
    }
    else
    {
        self.canRegisterUser = true;
    }
}

//---
- (BOOL)xmppStream:(XMPPStream *)sender didReceiveIQ:(XMPPIQ *)iq {
    
    return NO;
    
}

//When a message from server is received.
//Some messages are system kind or others than chat messages.
- (void)xmppStream:(XMPPStream *)sender didReceiveMessage:(XMPPMessage *)message {
    
    
    NSString *msg = [[message elementForName:@"body"] stringValue];
    NSString *from = [[message attributeForName:@"from"] stringValue];
    
    //From, must conform a contact name at contacts and chats lists.
    from = [from stringByReplacingOccurrencesOfString:mainConfig.serverShortName withString:@""];
    if ((msg != nil)&&(from != nil))
    {
        if (chatsDelegate != nil) [chatsDelegate NewMessage:msg fromUser:from];
        if (chatsViewDelegate != nil) [chatsViewDelegate NewChat];
    }
}

//Some presence event.
//Not all listes. Open/Close not aplied here.
- (void)xmppStream:(XMPPStream *)sender didReceivePresence:(XMPPPresence *)presence {
    
    NSString *presenceType = [presence type]; // online/offline
    NSString *myUsername = [[sender myJID] user];
    XMPPJID *fromUser = [presence from];
    NSString *presenceFromUser = [fromUser user];
    
    if (![presenceFromUser isEqualToString:myUsername]) {
        
        if ([presenceType isEqualToString:@"available"]) {
            
            [contactsDelegate UserOnline:[NSString stringWithFormat:@"%@@%@", presenceFromUser, mainConfig.serverFullName]];
            [contactsViewDelegate UserOnline:[NSString stringWithFormat:@"%@@%@", presenceFromUser, mainConfig.serverFullName]];
            
        } else if ([presenceType isEqualToString:@"unavailable"]) {
            
            [contactsDelegate UserOffline:[NSString stringWithFormat:@"%@@%@", presenceFromUser, mainConfig.serverFullName]];
            [contactsViewDelegate UserOffline:[NSString stringWithFormat:@"%@@%@", presenceFromUser, mainConfig.serverFullName]];
        }
        
    }
    
}


@end
