//
//  ChatsManager.m
//  rrrt
//
//  Created by Silvia Cabezas Nieto on 10/05/15.
//  Copyright (c) 2015 Silvia Cabezas Nieto. All rights reserved.
//
//
/* This model class represents the state of chats in the application. It's a direct dependence to it.
 Manages chats storage and serves them as requested.
 Can suscribe and unsuscribe views to chats directly (linker factory?)
 It's the proxy at message reception. All them comes through.
 */
//

#import "ChatsManager.h"

@implementation ChatsManager

@synthesize chats;

-(id)init
{
    self = [super init];
    chats = [[NSMutableArray alloc ] init];
    return self;
}

#pragma mark -
#pragma mark Chat delegate

- (void)NewMessage:(NSString *)message fromUser:(NSString *)name
{
    //Look for the chat
    TFCChat *chat = [self SearchChat:name];
    
    if (chat == nil) //Found?
    {
        //Create a new chat
        chat = [self CreateChat:name];
    }
    
    //Add the message to the chat
    TFCMessage *oMessage = [[TFCMessage alloc] initWithContentAndUser:message theUser:name];
    [chat AddMessage:oMessage];
}

- (TFCChat *)SearchChat:(NSString *)name
{
    for (TFCChat *chat in chats)
    {
        if ([chat.user isEqualToString:name])
        {
            return chat;
        }
    }
    return nil;
}

- (TFCChat *)CreateChat:(NSString *)name
{
    TFCChat *chat = [[TFCChat alloc] initWithUser:name];
    [chats addObject:chat];
    return chat;
}

- (TFCChat *)setActiveChatSubscription:(NSString *)name theView:(id)oView
{
    //Look for the chat
    TFCChat *chat = [self SearchChat:name];
    
    if (chat == nil) //Found?
    {
        //Create a new chat
        chat = [self CreateChat:name];
    }
    chat.delegateView = oView;
    [chat Readed];
    return chat;
}

- (void)clearActiveChatSubscription:(NSString *)name
{
    //Look for the chat
    TFCChat *chat = [self SearchChat:name];
    chat.delegateView = nil;
}

@end

